import EventTarget from "./EventTarget";
import { PeerInfo } from "./PeerInfo";
import WebRTC from "./WebRTC";
import MWSE from "./index";

interface IPeerOptions{

};


export default class Peer extends EventTarget
{
    public mwse : MWSE;
    public options : IPeerOptions = {};
    public socketId? : string;
    public selfSocket : boolean = false;
    public active : boolean = false;
    public info : PeerInfo;
    public rtc? : WebRTC;
    public peerConnection : boolean = false;
    public primaryChannel : "websocket" | "datachannel" = "datachannel";
    constructor(wsts:MWSE){
        super();
        this.mwse = wsts;
        this.info = new PeerInfo(this);
    }
    public createRTC() : WebRTC
    {
        this.rtc = new WebRTC();
        this.rtc.on("connected", () => {
            this.peerConnection = true;
        });
        this.rtc.on('disconnected', () => {
            this.peerConnection = false;
        })
        this.rtc.on("output",(payload:object) => {
            this.send({
                type: 'rtc:session',
                payload: payload
            })
        });
        this.on('message',(data:{type?:string,payload?:any}) => {
            if(data.type == 'rtc:session')
            {
                if(this.rtc)
                {
                    this.rtc.emit("input", data.payload)
                }
            }else if(data.type == 'rtc:message')
            {
                this.emit("message", data.payload)
            }
        });
        return this.rtc;
    }
    public setPeerOptions(options: string | IPeerOptions){
        if(typeof options == "string")
        {
            this.setSocketId(options)
        }else{
            this.options = options;
        }
    }
    public setSocketId(uuid: string){
        this.socketId = uuid;
    }
    async metadata() : Promise<any>
    {
        if(this.socketId == 'me')
        {
            let result = await this.mwse.EventPooling.request({
                type:'my/socketid'
            });
            this.selfSocket = true;
            this.active ||= true;
            this.socketId = result;
            this.emit('scope');
            this.activeScope = true;
            return result;
        }
    };
    
    async request(pack:any){
        if(this.active)
        {
           return await this.mwse.request(this.socketId as string, pack);
        }
    };
    equalTo(peer : Peer | {socketId: string})
    {
        return this.socketId == peer.socketId;
    }
    async isReachable()
    {
        return await this.mwse.EventPooling.request({
            type:'is/reachable',
            to: this.socketId
        });
    }
    async enablePairAuth(){
        await this.mwse.EventPooling.request({
            type:'auth/pair-system',
            value: 'everybody'
        });
    }
    async disablePairAuth(){
        await this.mwse.EventPooling.request({
            type:'auth/pair-system',
            value: 'disable'
        });
    }
    async send(pack: any){
        let isOpenedP2P = this.peerConnection;
        let isOpenedServer = this.mwse.server.connected;
        let sendChannel : "websocket" | "datachannel";
        if(isOpenedP2P && isOpenedServer)
        {
            if(this.primaryChannel == "websocket")
            {
                sendChannel = "websocket"
            }else
            {
                sendChannel = "datachannel"
            }
        }else if(isOpenedServer){
            sendChannel = "websocket"
        }else{
            sendChannel = "datachannel"
        }

        if(sendChannel == "websocket")
        {
            await this.mwse.EventPooling.request({
                type:'pack/to',
                pack,
                to: this.socketId
            });
        }else{
            this.rtc?.send({
                type: 'rtc:message',
                payload: pack
            })
        }
    }
    async forget(){
        this.mwse.peers.delete(this.socketId as string);
        this.mwse.pairs.delete(this.socketId as string);
    }
}   